/*
 * File name: main.cc
 * Date:      2018/10/02 23:52
 * Author:    Miroslav Kulich
 */

#include <stdio.h>

#include "robot_client/robot_client.h"

CRobotClient robot;
CLaserSensorConfiguration lacfg;

void control(CLaserScan &scan) {
  // Calculation of array size from angle range and angle increment.
  int size = scan.range.size();
  int minIndexLeft = size/2 -30;
  int minIndexRight = size / 2;

  // This cycle goes through array and finds minimum on the left and right
  for (int i = size/2 -30; i < size/2; i++) {
    if (scan.range[i] < scan.range[minIndexRight] && scan.range[i] > 0.05) {
      minIndexRight = i;
    }
  }
  for (int i = size/2; i < size/2 + 30; i++) {
    if (scan.range[i] < scan.range[minIndexLeft] && scan.range[i] > 0.05) {
      minIndexLeft = i;
    }
  }

  // Calculation of angle from indexes
  double angleMinLeft = (minIndexLeft - size / 2) * lacfg.angularResolution;
  double distMinLeft = scan.range[minIndexLeft];
  double angleMinRight = (minIndexRight - size / 2) * lacfg.angularResolution;
  double distMinRight = scan.range[minIndexRight];
  const double ANGLE_COEF = 1;    //!< Coeficient for angles.
  const double ROBOT_SPEED = 0.1;  //!< Speed of robot [m/s].

  // Sending info about processed data
  std::cout << "distLeft = " << distMinLeft << " distRight = " << distMinRight
            << std::endl;

  if (distMinLeft > 3 && distMinRight > 3) {
    std::cout << "Going straight" << std::endl;
    robot.setVelocities(0.1, 0.0);
  } else {
    double linear, angular;
    if (distMinLeft >= distMinRight) {
      angular = ANGLE_COEF * distMinLeft / distMinRight - ANGLE_COEF;
    } else {
      angular = -(ANGLE_COEF * distMinRight / distMinLeft - ANGLE_COEF);
    }

    linear = ROBOT_SPEED;
    if (distMinLeft < 0.25 && distMinRight < 0.25 && angleMinLeft < 0.7 &&
        angleMinRight < 0.7) {
      angular = 0.1;
      linear = 0;
	    std::cout << "Too close ... "<< std::endl;
    }
    std::cout << "Speed: " << linear << ", " << angular << std::endl;
    robot.setVelocities(linear, 0.2*angular);
  }
}

int main(int argc, char **argv) {
  CRobotClientConfiguration cfg;
  cfg.waitForInitialization = true;
  // robot.setAsyncMode();
  robot.enableRequestResponseServices();
  if (!robot.initialize(&cfg)) {
    std::cerr << "Failed to initialize robot!" << std::endl;
    return 1;
  }
  std::cerr << "Robot initialized." << std::endl;

  // bool res = robot.getLaserConfiguration(lacfg);
  bool res = robot.getLaserSensorConfiguration(lacfg);

  std::cout << "Res " << res << std::endl;
  std::cout << "Range " << lacfg.maxRange << std::endl;
  std::cout << "Angle resolution " << lacfg.angularResolution << std::endl;
  std::cout << "Min angle " << lacfg.minAngle << std::endl;
  std::cout << "Count " << lacfg.scanSampleCount << std::endl;
  std::cout << "Frequency " << lacfg.frequency << std::endl;

  CPose pose;
  CLaserScan scan;
  /*
  double x1, y1, x2, y2;

                  while(!robot.isFresh(ROBOT_DATA_ODOMETRY)) {
                                                  usleep(1000);
                  }

                  if (robot.getOdometry(pose)) {
                          x1 = pose.x;
                          y1 = pose.y;
                           std::cout <<  "odometry: [" << pose.x <<", " <<
  pose.y << ", " << pose.heading << "]" << std::endl;
                  }

          robot.setVelocities(0.0, 0.1);
          sleep(10);
          robot.setVelocities(0.0, 0.0);

                  while(!robot.isFresh(ROBOT_DATA_ODOMETRY)) {
                                                  usleep(1000);
                  }

                  if (robot.getOdometry(pose)) {
                          x2 = pose.x;
                          y2 = pose.y;
                           std::cout <<  "odometry: [" << pose.x <<", " <<
  pose.y << ", " << pose.heading << "]" << std::endl;
                  }

  std::cout << "Distance: " << sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)) <<
  std::endl;

          for(int i = 0; i<100;i++) {
                   robot.setVelocities(0.0, 0.0);
                                                  usleep(1000);
           }

  */
  while (1) {
    bool fresh = false;
    if (robot.isFresh(ROBOT_DATA_ODOMETRY)) {
      if (robot.getOdometry(pose)) {
        // std::cout <<  "odometry: [" << pose.x <<", " << pose.y << ", " <<
        // pose.heading << "]" << std::endl;
      }
      fresh = true;
    }
    if (robot.isFresh(ROBOT_DATA_LASER)) {
      if (robot.getLaserSensorData(scan)) {
        // std::cout << "scan " << scan.range.size() << std::endl;
        control(scan);
      }
      fresh = true;
    }
    if (!fresh) {
      fprintf(stdout, "nothing fresh\n");
      usleep(100000);
    }
  }

  return 0;
}
